; ***************************
; * Example: Move a sprite  *
; ***************************
 
*=$0801   ; Starting Address BASIC + 1 => SYS 2049
 
        byte $0C,$08,$40,$00,$9E,$20,$32,$30,$36,$32,$00,$00,$00 ; BASIC CODE: 1024 SYS 2062
  
  ; Define some Constants for convenience
COLOR_BLACK       = $00
COLOR_GREEN       = $05
COLOR_LIGHTGREEN  = $0D

SPRITE_0_X_POSITION = $D000
SPRITE_0_Y_POSITION = $D001

BORDER_COLOR        = $D020
BACKGROUND_COLOR    = $D021

SPRITE_0_COLOR      = $D027

SPRITE_0_POINTER    = $0400 + $03F8 ; Last 8 Bytes of Screen RAM
SPRITE_0_DATA       = $0340         ; Block 13, 13*64=>832 => $0340

SPRITE_UPPER_X      = $D010


        ; Start Program
        jsr $E544             ; call the Function that clears the screen

        LDA #$0D              ; using block 13 for sprite0
        STA SPRITE_0_POINTER  ; set block 13 as target address for Data of Sprite0

        ;; Initialization
        lda #COLOR_GREEN      ; load green color code into A
        sta BORDER_COLOR      ; make border green

        lda #COLOR_LIGHTGREEN ; load lightgreen color code into A
        sta BACKGROUND_COLOR  ; make background lightgreen

        lda #$01              ; enable...
        sta $D015             ; ...Sprite 0 => %0000 0001 (all sprites off except Sprite 0)

        lda #COLOR_BLACK      ; load black color code into A
        sta SPRITE_0_COLOR    ; make Sprite0 completely black

        ; Reset Sprite Data
        ldx #$00    ; init x
        lda #$00    ; init a

clean
        sta SPRITE_0_DATA,x   ; write 0 into sprite data at x
        inx                   ; increment x
        cpx #$3F              ; is x <= 63?
        bne clean             ; if yes, goto clean

; Build the Sprite
        ldx #$00              ; init x
build
        lda data, x           ; load data at x
        sta SPRITE_0_DATA,x   ; write into sprite data at x
        inx                   ; increment x
        cpx #$3F              ; is x <= 63?
        bne build             ; if yes, goto build

; Set Start Location of Sprite 0
ldx #$64                  ; initial x position = 100
        ldy #$46                  ; initial y position =  70
        stx SPRITE_0_X_POSITION   ; move sprite 0 to x position
        sty SPRITE_0_Y_POSITION   ; move sprite 0 to y position

; Do some Movement
move_down:
iny                       ; increment y position  
sty SPRITE_0_Y_POSITION   ; move sprite 0 to y position
jsr wait                  ; call wait subroutine
cpy #$C8                  ; are we at y=200?
bne move_down             ; if no, go to move_down

move_up:
dey                       ; decrement y position  
sty SPRITE_0_Y_POSITION   ; move sprite 0 to y position
jsr wait                  ; call wait subroutine
cpy #$46                  ; are we at y=70?
bne move_up               ; if no, go to move_up  

move_right:
inx                       ; increment x position
stx SPRITE_0_X_POSITION   ; move sprite 0 to x position
jsr wait                  ; call wait subroutine
cpx #255                  ; are we at x=255?
bne move_right            ; if no, goto move_right

lda #$01                  ; BITMASK %0000 0001
ora SPRITE_UPPER_X        ; set bit 0 to 1 (leave others unchanged)
sta SPRITE_UPPER_X        ; write result to memory

move_right_end:
inx                       ; increment x position
stx SPRITE_0_X_POSITION   ; move sprite 0 to x position
jsr wait                  ; call wait subroutine
cpx #$2D                  ; are we at 255+45 = 300?
bne move_right_end        ; if no, goto move_right_end

move_left:
dex                       ; decrement x position
stx SPRITE_0_X_POSITION   ; move sprite 0 to x position
jsr wait                  ; call wait subroutine
cpx #$00                  ; are we at x=0?
bne move_left             ; if no, goto move_left

lda #$FE                  ; BITMASK %1111 1110
and SPRITE_UPPER_X        ; set bit 0 to 0 (leave others unchanged)
sta SPRITE_UPPER_X        ; write result to memory

move_left_end:  
dex                       ; decrement x position
stx SPRITE_0_X_POSITION   ; move sprite 0 to x position
jsr wait                  ; call wait subroutine
cpx #$64                  ; are we at x=100?
bne move_left_end         ; if no, go to move_left_end

; Wait Subroutine
;; This will make the c64 wait for roughly one frame
wait:
lda #$FF    ; load 255 into A
cmp $D012   ; look if current rasterline equals 255
bne wait    ; if no, goto wait
rts         ; if yes, return from subroutine

; The Data for our Sprite
data:
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255
        byte 255, 255, 255